#!/usr/bin/python
# -*- coding: latin-1 -*-

# Reads results from the value analysis and computes a map of which intervals
# have been bounded

import sys
import re
import parse_interval
from parse_interval import *
import os
import os.path

print_csv = len(sys.argv) > 1 and sys.argv[1] == "-csv"

# "main" starts here

# First, we read all input lines
all_input_lines = [line.strip() for line in sys.stdin.readlines()]

# Then, we separate those containing "line indicators", prefixed with '@@'
all_lines_with_numbers = [line for line in all_input_lines if "@@" in line]

# We use them to construct the map of "default results".
# NOTE: it is risky to initialize the map with "Y" for each line,
# since an error might mean all lines will be considered bounded,
# but in case the line is not executed (for instance, if the analyzer
# detects it is dead code), this allows the line to be correctly counted.

bounded_map = {}
for line in all_lines_with_numbers:
  line_number = int(line[2:])
  print "line_number: %d" % line_number
  bounded_map[line_number] = 1 # Bounded by default

# Now, we simply select all lines containing interval annotations,
# and process them
lines = [line for line in all_input_lines if "##" in line]

unions = calculate_unions(lines)

all_lines_vars = unions.keys()
count_bounded = 0
count_intervals = 0
for key in all_lines_vars:
  (line_number, var) = key
  count_intervals += 1
  # Note: these unions[i] variables are strings, not intervals
  union_str = unions[key] if key in unions else []

  (uitvs, modulo, _) = parse_union(union_str)
  uitv = itv_union(uitvs)
  len_itv = itv_len(uitv, modulo)
  len_bounded = yes_no(is_len_bounded(len_itv))
  #sys.stderr.write("itv: %s - len: %ld - bounded: %s\n" % (uitv, len_itv, len_bounded))
  res = 1 if len_bounded == "Y" else 0
  count_bounded += res
  print "line_number_: %d" % int(line_number)
  bounded_map[int(line_number)] = res
  
  if print_csv:
    print '>>>;%s;%s;%s;"Our analyzer";"%s";%ld;%s' % (bprefix, line_number, var, union_to_str(union_str), len_itv, len_bounded)
  else:
    format_str = "##%-4s: %5s: %-38s | %s"
    print (format_str) % (line_number, var, union_to_str(union_str), len_bounded)

# Prints each result line preceded by "@@RES" for it to be easily grepped by a script
for line_number in sorted(bounded_map.keys()):
  print "@@RES:%4s:%d" % (line_number, bounded_map[line_number])

print "Total bounded variables: %3d / %3d (%5.1f%%)" % (count_bounded, count_intervals, 100.0*count_bounded/count_intervals)
