#!/bin/bash

# Checks if the files specified do not contain two DUMPs in a same line number.
# For programs containing multiple C source files 
# (e.g., main.c file1.c file2.c), you must check
# that no DUMP statements are placed in a same line
# number across all files
# (e.g., if both file1.c and file2.c contain
# a DUMP statement in line 42, the counting scripts
# will fail).
# Prints the number of conflicting dumps and the file
# names and lines where these conflicts happen

if [ $# -lt 1 ]
then
  echo "Syntax: $0 [ <file.c> ]+ "
  echo "  Warning: do not use '*' as"
  echo "  "
  exit 1
fi


OFFENDERS=($(grep --with-filename -n DUMP $* | cut -d":" -f2 | sort -n | uniq -c | grep -v "^ *1 "))

echo "Number of conflicting DUMPs: ${#OFFENDERS[@]}"

for off in ${OFFENDERS[@]}
do
  grep --with-filename -n DUMP $* | grep ":$off:"
done