#!/bin/bash

# Uses the result of a value analysis (on a .res file) to compute
# the number of bounded and total intervals for the file.

# Syntax: $0 <file.c> <file.res>

if [ $# -lt 2 ]
then
  echo "Syntax: $0 <file.c> <file.res>"
  exit 2
fi

if [ ! -r "$1" ]
then
  echo "Error: cannot read input source file: $1"
  exit 1
fi

if [ ! -r "$2" ]
then
  echo "Error: cannot read input .res file: $2"
  exit 1
fi

f=$1
res=$2

SCR_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
PROG=${f%.c}
BPROG=$(basename $PROG)

# To obtain the total number of DUMPs, we use the special compilation variable
# PRINT_DUMP_LINES and the preprocessor to ensure that DUMPs in
# multi-source programs are correctly counted.
NUMDUMPS=$(gcc -E -DPRINT_DUMP_LINES $f | grep ":DUMP" | wc -l)

# We perform several steps:
# - obtain the line numbers of the DUMP annotations, prefix them with @@;
# - send both line numbers and the result of the analysis to a script;
#   this script outputs the result "<line>:1" for each line that has been
#   bounded, and "<line>:0" for each line not bounded;
# - process the result to count how many 1's have been found;
# - for each program, output its name, the number of bounded variables
#   and the total annotation count
gcc -E -DPRINT_DUMP_LINES $f | grep ":DUMP" | sed 's/.*(\([0-9]\+\):.*).*/\1/' | \
  cut -d ":" -f 1 | sed 's/^/@@/' | \
  cat - $res | $SCR_DIR/calc_bounded_intervals.py | \
  grep '^@@RES:' | sed 's/^@@RES://' | \
  cut -d ":" -f 2 | \
  grep 1 | wc -l | \
  sed "s|^|${BPROG}:|" | sed "s|$|:${NUMDUMPS}|"