(* *********************************************************************)
(*                                                                     *)
(*              The Compcert verified compiler                         *)
(*                                                                     *)
(*          Xavier Leroy, INRIA Paris-Rocquencourt                     *)
(*                                                                     *)
(*  Copyright Institut National de Recherche en Informatique et en     *)
(*  Automatique.  All rights reserved.  This file is distributed       *)
(*  under the terms of the INRIA Non-Commercial License Agreement.     *)
(*                                                                     *)
(* *********************************************************************)

open Array
open Datatypes
open AST
open Maps
open CFG
open Camlcoq
open BinPos

let should_inline (id: ident) (f: coq_function) =
  (* Default heuristics: 'inline' modifier *)
  (*C2C.atom_is_inline id*)

  (* Value analysis heuristics: everything but main *)
  extern_atom id <> "main"


(* Copied from PrintCminor *)
let ident_name id =
  match id with
  | Coq_xO n -> extern_atom n
  | Coq_xI n -> Printf.sprintf "$%ld" (camlint_of_positive n)
  | Coq_xH -> "$0"

(* Associates [new_id] with the string representation of
   [old_id] plus a suffix. If [old_id] is not currently
   associated to any identifier, assumes it is a Cminor
   temporary (e. g. "$1") and uses this as its string
   representation.
   [new_id] must not be associated to any string. *)
let assoc_id (old_id: ident) (new_id: ident) : unit =
  let s = (ident_name old_id) ^ "_1" in
  try
    let s' = Hashtbl.find string_of_atom new_id in
    failwith (Printf.sprintf "Assertion error: new_id %s already associated (%s)" (ToString.(~$) new_id) s')
  with Not_found ->
    (* new_id is available *)
    Hashtbl.add atom_of_string s new_id;
    Hashtbl.add string_of_atom new_id s
