open Printf
open Errors

(* Printing of error messages *)

(* Locally named idents are encoded as in Cminorgen.
   This function should live somewhere more central. *)
let ident_name id =
   match id with
  | BinPos.Coq_xO n -> Camlcoq.extern_atom n
  | BinPos.Coq_xI n -> Printf.sprintf "$%ld" (Camlcoq.camlint_of_positive n)
  | BinPos.Coq_xH -> "$0"

let print_error oc msg =
  let print_one_error = function
  | Errors.MSG s -> output_string oc (Camlcoq.camlstring_of_coqstring s)
  | Errors.CTX i -> output_string oc (Camlcoq.extern_atom i)
  | Errors.CTXL i -> output_string oc (ident_name i)
  in
    List.iter print_one_error msg;
    output_char oc '\n'

let do_res (r : 'a res) : 'a = 
  match r with
    | OK v -> v
    | Error msg ->
      print_error stderr msg;
      exit 2

let do_opt (o : 'a option) : 'a = 
  match o with
    | Some a -> a
    | None ->
      fprintf stderr "do_opt: None\n";
      exit 2

let get_main (rtl_prog : RTL.program) : (AST.ident * RTL.coq_function) =
  let main_id = rtl_prog.AST.prog_main in
  let main_fd = List.assoc main_id rtl_prog.AST.prog_funct in
  match main_fd with
    | AST.External _ -> failwith "Assertion error: main is external"
    | AST.Internal main_f -> (main_id, main_f)

(* If the program has no main function, raises Not_found. *)
let cfg_get_main (cfg_prog : CFG.program) : (AST.ident * CFG.coq_function) =
  let main_id = cfg_prog.AST.prog_main in
  let main_fd = List.assoc main_id cfg_prog.AST.prog_funct in
  match main_fd with
    | AST.External _ -> failwith "Assertion error: main is external"
    | AST.Internal main_f -> (main_id, main_f)
      
(** val transf_c_program_to_rtl : Csyntax.program -> program Errors.res **)
open Compiler
let transf_c_program_to_unoptimized_rtl p : RTL.program res =
  apply_partial
    (apply_total
       (apply_total
          (apply_partial
             (apply_partial
                (apply_total
                   (apply_partial (Errors.OK (Obj.magic p))
                      SimplExpr.transl_program) (print print_Clight))
                Cshmgen.transl_program)
             Cminorgen.transl_program)
          (print print_Cminor)) Selection.sel_program)
    RTLgen.transl_program

let transf_c_program_to_inlined_rtl p : RTL.program res =
  apply_partial
    (apply_partial
       (apply_total
          (apply_total
             (apply_partial
                (apply_partial
                   (apply_total
                      (apply_partial (Errors.OK (Obj.magic p))
                         SimplExpr.transl_program) (print print_Clight))
                   Cshmgen.transl_program)
                Cminorgen.transl_program)
             (print print_Cminor)) Selection.sel_program)
       RTLgen.transl_program)
    Inlining.transf_program

let transf_c_program_to_cfg p : CFG.program res =
  (apply_partial
     (apply_total
        (apply_partial
           (apply_partial
              (apply_total
                 (apply_partial (Errors.OK (Obj.magic p))
                    SimplExpr.transl_program) (print print_Clight))
              Cshmgen.transl_program)
           Cminorgen.transl_program)
        (print print_Cminor)) 
     CFGgen.transl_program)

let transf_c_program_to_inlined_cfg p : CFG.program res =
  apply_partial (transf_c_program_to_cfg p) InliningCFG.transf_program

let transf_c_program_to_renumbered_cfg p : CFG.program res =
  apply_total (transf_c_program_to_inlined_cfg p) RenumberCFG.transf_program
