(* Ce fichier est utilisé par SliceToString, donc il ne peut pas l'inclure *)

open BinPos
open BinNat
open Maps
open Camlcoq
open CFG
open BourdoncleIterator
module B = Bourdoncle

type node = positive

let int_of_positive p =
  let i = camlint_of_positive p in
    (Int32.to_int i) -1
      
let rec positive_of_int n =
  if n = 0 then assert false else
  if n = 1 then Coq_xH else
  if n land 1 = 0
  then Coq_xO (positive_of_int (n lsr 1))
  else Coq_xI (positive_of_int (n lsr 1))
let positive_of_int n = positive_of_int (n+1)

(* Fonctions recopiées de SliceToString *)
let nodeToString (p : positive) : string =
  Int32.to_string (Camlcoq.camlint_of_positive p)
let rec cleanListToString' (aToString: 'a -> string) (l : 'a list) =
  match l with
  | [] -> ""
  | e :: r -> " " ^ (aToString e) ^ (cleanListToString' aToString r)
let cleanListToString (aToString: 'a -> string) (l : 'a list) =
  match l with
  | [] -> "[]"
  | [e] -> "(" ^ (aToString e) ^ ")"
  | e :: r -> "(" ^ (aToString e) ^ (cleanListToString' aToString r) ^ ")"
let rec bourdoncleToString (b : B.bourdoncle) : string =
  match b with
    | B.I n -> (nodeToString n)
    | B.L (h, lb) -> cleanListToString bourdoncleToString ((B.I h) :: lb)
let bourdoncleListToString (l : B.bourdoncle list) : string =
  cleanListToString bourdoncleToString l

(* Type bidon juste pour éviter de changer les types des fonctions déjà définies *)
type instr = | Inop

let build_cfg f =
  let entry = int_of_positive f.fn_entrypoint in
  let max = PTree.fold (fun m n _ -> max m (int_of_positive n)) f.fn_code 0 in
  (* nodes are between 1 and max+1 *)
  let succ = Array.make (max+1) [] in
  let _ = PTree.fold (fun () n ins ->
			succ.(int_of_positive n) <- 
			  match ins with
			    | Iskip j -> [(int_of_positive j, Inop)]
			    | Iassign (_,_,j) -> [(int_of_positive j, Inop)]
			    | Istore (_,_,_,j) -> [(int_of_positive j, Inop)]
			    | Icall (_,_,_,_,j) -> [(int_of_positive j, Inop)]
			    | Itailcall _ -> []
			    | Ibuiltin (_,_,_,j) -> [(int_of_positive j, Inop)]
			    | Iifthenelse (_,j,k) -> [(int_of_positive j, Inop);(int_of_positive k, Inop)]
			    | Iswitch (_,tbl,def) -> (int_of_positive def, Inop) :: List.map (fun (_,j) -> (int_of_positive j, Inop)) tbl
			    | Ireturn _ -> []) f.fn_code () in
    { entry = entry; succ = succ }

let rec build_bourdoncle' (bl : bourdoncle list) : B.bourdoncle list =
  match bl with
    | [] -> []
    | (I i) :: r -> B.I (positive_of_int i) :: (build_bourdoncle' r)
    | (L ((I h) :: l, _)) :: r -> B.L (positive_of_int h, build_bourdoncle' l) :: (build_bourdoncle' r)
    | _ -> failwith "Assertion error: invalid bourdoncle ist"

let build_bourdoncle'' (f : coq_function) : B.bourdoncle =
  let cfg = build_cfg f in
  match build_bourdoncle' (get_bourdoncle cfg) with
    | [] -> failwith "assertion error: empty bourdoncle"
    | l -> 
      begin
        match l with
          | B.I h :: r -> B.L (h, r)
          | B.L (_, _) :: _ -> 
            begin
              Printf.printf "ASSERTION ERROR: invalid program structure (too many bourdoncles)\n";
              Printf.printf "Head should be an element, but it is a list\n";
              Printf.printf "Failed at: %s\n" (bourdoncleListToString l);
              failwith "assertion error"
            end
          | _ -> failwith "assertion error : ???"
      end


(* Fonction auxiliaire pour build_order *)
let rec linearize (b : B.bourdoncle) : node list =
  match b with
    | B.I n -> [n]
    | B.L (h, l) -> List.fold_left (fun l0 b' -> l0 @ (linearize b')) [h] l

let rec build_order' (l : node list) (count : coq_N) : coq_N PTree.t =
  match l with
    | [] -> PTree.empty
    | n :: r -> PTree.set n count (build_order' r (coq_Nsucc count))
  
let build_order (b : B.bourdoncle) : coq_N PMap.t =
  let bo = build_order' (linearize b) (coq_Nsucc N0) in
  (coq_Nsucc N0, bo)
(*  fun n ->
    match PTree.get n bo with
      | None -> (coq_Nsucc N0)
      | Some n' -> n'*)

let build_bourdoncle (f : coq_function) : (B.bourdoncle * coq_N PMap.t) =
  let b = build_bourdoncle'' f in
  let bo = build_order b in
  (b, bo)
